@@ -1,5 +1,5 @@
 package Inline::C;
-$Inline::C::VERSION = '0.50';
+$Inline::C::VERSION = '0.53';
 $Inline::C::VERSION = eval $Inline::C::VERSION;
 
 use strict;
@@ -45,7 +45,7 @@ sub validate {
     $o->{ILSM}{MAKEFILE} ||= {};
     if (not $o->UNTAINT) {
 	require FindBin;
-	$o->{ILSM}{MAKEFILE}{INC} = "-I\"$FindBin::Bin\"";
+	$o->{ILSM}{MAKEFILE}{INC} = "-I\"$FindBin::Bin\"" if not defined $o->{ILSM}{MAKEFILE}{INC};
     }
     $o->{ILSM}{AUTOWRAP} = 0 if not defined $o->{ILSM}{AUTOWRAP};
     $o->{ILSM}{XSMODE} = 0 if not defined $o->{ILSM}{XSMODE};
@@ -65,6 +65,19 @@ END
 
     while (@_) {
 	my ($key, $value) = (shift, shift);
+      if ($key eq 'PRE_HEAD') {
+         unless( -f $value) {
+           $o->{ILSM}{AUTO_INCLUDE} = $value . "\n" . $o->{ILSM}{AUTO_INCLUDE};
+         }
+         else {
+           my $insert;
+           open RD, '<', $value or die "Couldn't open $value for reading: $!";
+           while(<RD>) {$insert .= $_}
+           close RD or die "Couldn't close $value after reading: $!";
+           $o->{ILSM}{AUTO_INCLUDE} = $insert . "\n" . $o->{ILSM}{AUTO_INCLUDE};
+         }
+         next;
+      }
 	if ($key eq 'MAKE' or
 	    $key eq 'AUTOWRAP' or
             $key eq 'XSMODE'
@@ -92,6 +105,10 @@ END
 	    $o->add_string($o->{ILSM}{MAKEFILE}, $key, $value, '');
 	    next;
 	}
+      if ($key eq 'CCFLAGSEX') {
+	    $o->add_string($o->{ILSM}{MAKEFILE}, 'CCFLAGS', $Config{ccflags} . ' ' . $value, '');
+          next;
+      }
 	if ($key eq 'TYPEMAPS') {
           unless(ref($value) eq 'ARRAY') {
 	      croak "TYPEMAPS file '$value' not found"
@@ -744,10 +761,15 @@ sub compile {
     ($cwd) = $cwd =~ /(.*)/ if $o->UNTAINT;
 
     chdir $build_dir;
-    $o->call('makefile_pl', '"perl Makefile.PL"', 2);
-    $o->call('make', '"make"', 2);
-    $o->call('make_install', '"make install"', 2);
+    # Run these in an eval block, so that we get to chdir back to
+    # $cwd if there's a failure. (Ticket #81375.)
+    eval {
+      $o->call('makefile_pl', '"perl Makefile.PL"', 2);
+      $o->call('make', '"make"', 2);
+      $o->call('make_install', '"make install"', 2);
+    };
     chdir $cwd;
+    die if $@; #Die now that we've done the chdir back to $cwd. (#81375)
     $o->call('cleanup', 'Cleaning Up', 2);
 }
 
@@ -869,8 +891,75 @@ sub fix_make {
 }
 
 sub quote_space {
-    $_[0] = '"'.$_[0].'"' if $_[0] and $_[0] =~ /\s/;
-    $_[0];
+
+  # Do nothing if $ENV{NO_INSANE_DIRNAMES} is set
+  return $_[0] if $ENV{NO_INSANE_DIRNAMES};
+
+  # If $_[0] contains one or more doublequote characters, assume
+  # that whitespace has already been quoted as required. Hence,
+  # do nothing other than immediately return $_[0] as is.
+  # We currently don't properly handle tabs either, so we'll
+  # do the same if $_[0] =~ /\t/.
+  return $_[0] if ($_[0] =~ /"/ || $_[0] =~ /\t/);
+
+  # We want to split on /\s\-I/ not /\-I/
+  my @in = split /\s\-I/, $_[0];
+  my $s = @in - 1;
+  my %s;
+  my %q;
+
+  # First up, let's reinstate the ' ' characters that split
+  # removed
+  for(my $i = 0; $i < $s; $i++) {
+    $in[$i] .= ' ';
+  }
+
+  # This for{} block dies if it finds that any of the ' -I'
+  # occurrences in $_[0] are part of a directory name.
+  for(my $i = 1; $i < $s; $i++) {
+    my $t = $in[$i + 1];
+    while($t =~ /\s$/) {chop $t}
+    die "Found a '", $in[$i], "-I", $t, "' directory.",
+        " INC Config argument is ambiguous.",
+        " Please use doublequotes to signify your intentions"
+      if -d ($in[$i] . "-I" . $t);
+  }
+
+  $s++; # Now the same as scalar(@in)
+
+  # Remove (but also Keep track of the amount of) whitespace
+  # at the end of each element of @in.
+  for(my $i = 0; $i < $s; $i++) {
+    my $count = 0;
+    while($in[$i] =~ /\s$/) {
+      chop $in[$i];
+      $count++;
+    }
+    $s{$i} = $count;
+  }
+
+  # Note which elements of @in still contain whitespace. These
+  # (and only these) elements will be quoted
+  for(my $i = 0; $i < $s; $i++) {
+    $q{$i} = 1 if $in[$i] =~ /\s/;
+  }
+
+  # Reinstate the occurrences of '-I' that were removed by split(),
+  # insert any quotes that are needed, reinstate the whitespace
+  # that was removed earlier, then join() the array back together
+  # again.
+  for(my $i = 0; $i < $s; $i++) {
+    $in[$i] = '-I' . $in[$i] if $i;
+    $in[$i] = '"' . $in[$i] . '"' if $q{$i};
+    $in[$i] .= ' ' x $s{$i};
+  }
+
+  # Note: If there was no whitespace that needed quoting, the
+  # original argument should not have changed in any way.
+
+  my $out = join '', @in;
+  $out =~ s/"\-I\s+\//"\-I\//g;
+  $_[0] = $out;
 }
 
 sub fix_space {
@@ -890,7 +979,6 @@ sub _parser_test {
     }
 
     warn $! if !open(TEST_FH, '>>', "$dir/parser_id");
-
     print TEST_FH $_[0];
     warn $! if !close(TEST_FH);
 }
