@@ -2,7 +2,7 @@ package Inline;
 
 use strict;
 require 5.006;
-$Inline::VERSION = '0.50';
+$Inline::VERSION = '0.53';
 $Inline::VERSION = eval $Inline::VERSION;
 
 use AutoLoader 'AUTOLOAD';
@@ -193,12 +193,11 @@ sub bind {
 # Process delayed objects that don't have source code yet.
 #==============================================================================
 # This code is an ugly hack because of the fact that you can't use an
-# INIT block at "run-time proper". So we kill the warning for 5.6+ users
-# and tell them to use a Inline->init() call if they run into problems. (rare)
-my $lexwarn = ($] >= 5.006) ? 'no warnings;' : '';
+# INIT block at "run-time proper". So we kill the warning and tell users
+# to use an Inline->init() call if they run into problems. (rare)
 
 eval <<END;
-$lexwarn
+no warnings;
 \$INIT = \$INIT; # Needed by Sarathy's patch.
 sub INIT {
     \$INIT++;
@@ -352,7 +351,7 @@ sub read_DATA {
 	local ($/);
 	my ($CR, $LF) = (&Socket::CR, &Socket::LF);
 	(my $data = <Inline::DATA>) =~ s/$CR?$LF/\n/g;
-	@{$DATA{$pkg}} = split /(?m)(__\S+?__\n)/, $data;
+	@{$DATA{$pkg}} = split /(?m)^[ \t]{0,}(__\S+?__\n)/, $data;
 	shift @{$DATA{$pkg}} unless ($ {$DATA{$pkg}}[0] || '') =~ /__\S+?__\n/;
     }
     ($marker, $o->{API}{code}) = splice @{$DATA{$pkg}}, 0, 2;
@@ -738,8 +737,11 @@ sub check_config_file {
            my $config = join '', <CONFIG>;
            close CONFIG;
 
-           croak M62_invalid_config_file(File::Spec->catfile($DIRECTORY,$configuration_file))
-             unless $config =~ /^version :/;
+           unless($config =~ /^version :/) {
+             warn "\$load_cfg sub: \$config: *${config}*\n";
+             croak M62_invalid_config_file(File::Spec->catfile($DIRECTORY,$configuration_file));
+           }
+
            if(UNTAINT) {
              warn "In Inline::check_config_file(): Blindly untainting Inline configuration file information.\n"
                unless $o->{CONFIG}{NO_UNTAINT_WARN};
@@ -855,6 +857,7 @@ sub create_config_file {
 
     my $file = File::Spec->catfile($ARGV[0], $configuration_file);
     open CONFIG, "> $file" or croak M24_open_for_output_failed($file);
+    flock(CONFIG, LOCK_EX);
     print CONFIG Inline::denter->new()
       ->indent(*version => $Inline::VERSION,
 	       *languages => \%languages,
@@ -862,6 +865,7 @@ sub create_config_file {
 	       *modules => \%modules,
 	       *suffixes => \%suffixes,
 	      );
+    flock(CONFIG, LOCK_UN);
     close CONFIG;
     exit 0;
 }
